#include <SoftwareSerial.h>
#include <Time.h>
#include "U8glib.h"

#define DEBUG false
#define syncInterval 60 * 60 //Sekunden
//Hardware
#define GPIO2 3     //ESP8266
#define GPIO0 4     //ESP8266
#define espReset 7  //ESP8266
#define espRX 13    //ESP8266
#define espTX 12    //ESP8266
#define gint 2      //Interrupt gesture Sensor

U8GLIB_64128N u8g(4,5, 1, 3, 2); 
SoftwareSerial ESP(espTX, 70); // RX, TX

// Variables
boolean readyCharOk1 = false;
boolean readyCharOk2 = false;
boolean readyCharOk3 = false;
boolean readyCharOk4 = false;
boolean readyCharOk5 = false;
byte x;
byte secondOld = 61;
unsigned long epoch;

void setup() {
  pinMode(GPIO0,OUTPUT);
  digitalWrite(GPIO0,HIGH);
  pinMode(GPIO2,OUTPUT);
  digitalWrite(GPIO2,HIGH);
  pinMode(espReset,OUTPUT);
  
  Serial.begin(9600);
  ESP.begin(9600);
  
  initEsp();
}

void loop() {
  
  
  if (secondOld != second()) {
    secondOld = second(); 
    drawTime();
    if (ESP.available()) epoch = readEspSerial();
  } 
}
void setStdFont() {
    //u8g.setFont(u8g_font_helvB14);
    u8g.setFont(u8g_font_fub14r);
}

void drawScreen(String x) {
  setStdFont();
  u8g.firstPage();  
  do {
    u8g.setPrintPos(0, 32); 
    u8g.print(x);
  } while( u8g.nextPage() );
}

void drawTime() {
  u8g.firstPage();  
  do {
    u8g.setPrintPos(5, 45); 
    u8g.setFont(u8g_font_fub25r);
    if (hour() < 10) u8g.print("  ");
    u8g.print(hour());
    u8g.print(":");
    u8g.setFont(u8g_font_fur14);
    u8g.print(" ");
    u8g.setFont(u8g_font_fub25r);
    if (minute() < 10) u8g.print("0");
    u8g.print(minute());
    u8g.setFont(u8g_font_fur14);
    u8g.print(" ");
    if (second() < 10) u8g.print("0");
    u8g.print(second());
  } while( u8g.nextPage() );
}
unsigned long readEspSerial() {
  unsigned long t = 0;
  byte x = 0;
  
  do {
    if (ESP.available()) {
      x = ESP.read();
      if (DEBUG) Serial.print(char(x));
      if ((x >= 48) && (x <= 57)) {
        t = 10*t + (x - 48);
      } // if zahl
    }  // if Daten an EPS
  } while (x!= 10);
  
  return t;
} // readEspSerial

boolean adjustDstEurope(unsigned long i)
{
 // last sunday of march
 int beginDSTDate=  (31 - (5* year(i) /4 + 4) % 7);
 int beginDSTMonth=3;
 //last sunday of october
 int endDSTDate= (31 - (5 * year(i) /4 + 1) % 7);
 int endDSTMonth=10;
 // DST is valid as:
 if (((month(i) > beginDSTMonth) && (month(i) < endDSTMonth))
     || ((month(i) == beginDSTMonth) && (day(i) >= beginDSTDate)) 
     || ((month(i) == endDSTMonth) && (day(i) <= endDSTDate)))
 return true;  // DST europe = utc +2 hour
 else return false; // nonDST europe = utc +1 hour
}

time_t getTimeFromEsp() {
  time_t epoch;
  digitalWrite(GPIO2,HIGH);
  epoch = readEspSerial();
  digitalWrite(GPIO2,LOW);
    
  if (adjustDstEurope(epoch)) epoch = epoch + (2 * 3600);
    else epoch = epoch + 3600;

  return epoch;
}

void initEsp() {
  digitalWrite(GPIO2,HIGH);
  delay(50);
  drawScreen("Reset ESP");
  digitalWrite(espReset,LOW);
  delay(100);
  digitalWrite(espReset,HIGH);

  drawScreen("Wait for WiFi"); 
  while (!(readyCharOk1 && readyCharOk2 && readyCharOk3 && readyCharOk4 && readyCharOk5)) {
    if (ESP.available()) {
      x = ESP.read();
      if (DEBUG) Serial.print(char(x));
      if (char(x) == 'R') readyCharOk1 = true;
      if (char(x) == 'E') readyCharOk2 = true;
      if (char(x) == 'A') readyCharOk3 = true;
      if (char(x) == 'D') readyCharOk4 = true;
      if (char(x) == 'Y') readyCharOk5 = true;
    }
  } //while
  do {
    if (ESP.available()) x = ESP.read();
    } while (x!= 10);

  drawScreen("WiFi Ready");
  if (DEBUG) Serial.println();
  if (DEBUG) Serial.println();
  
  epoch = readEspSerial();
  digitalWrite(GPIO2,LOW);
  if (epoch != 0) setTime(epoch);
  
  drawScreen("Wait for NTP");
  setSyncProvider(getTimeFromEsp);
  setSyncInterval(syncInterval);
}
